class GoldenIntegers(Ring):
    def __init__(self):
        Ring.__init__(self, base=ZZ)
        phi = self.hom([ RDF((1+sqrt(5))/2 ) ])
        psi = self.hom([CDF( (1+sqrt(5))/2 )])
        phi.register_as_coercion()
        psi.register_as_coercion()        
                
    def _is_valid_homomorphism_(self, codomain, im_gens):
        if len(im_gens) != 1: 
            return False
        if im_gens[0]**2 - im_gens[0] - 1 == 0: 
            return True
        return False
    
    def _repr_(self):
        return "The Golden Ring"
       
    def gen(self, i=0):    
        if i == 0:
            return GoldenElement(self, 0, 1)
        else:
            raise IndexError
               
    def ngens(self):
        return 1    
              
    def _element_constructor_(self, x):
        if isinstance(x, GoldenElement):
            return x
        return GoldenElement(self, ZZ(x), ZZ(0))
    
    def _coerce_map_from_(self, S):
        #print "_coerce_map_from_(%s, %s)"%(self, S)
        if S is ZZ:
            return True
        if S == Integers(13):
            return True    

class GoldenElement(RingElement):
    def __init__(self, parent, a, b):
        RingElement.__init__(self, parent)  # sets self._parent to parent
        self._a = a
        self._b = b

    def _im_gens_(self, codomain, im_gens):
        #print "_im_gens_(%s, %s, %s)"%(self, codomain, im_gens)
        return codomain(self._a) + codomain(self._b)*im_gens[0]

    def _repr_(self):
        return "%s + %s*(1+sqrt(5))/2"%(self._a, self._b)

    def _add_(left, right):
        return GoldenElement(left.parent(), left._a+right._a, left._b+right._b)

    def _sub_(left, right):
        return GoldenElement(left.parent(), left._a-right._a, left._b-right._b)

    def _mul_(left, right):
        raise NotImplementedError  
        
    def __float__(self):
        return float(1)
